@@ -20,14 +20,16 @@
 #ifndef VBA_FLASH_H
 #define VBA_FLASH_H
 
-extern void flashSaveGame(gzFile gzFile);
-extern void flashReadGame(gzFile gzFile, int version);
+extern void flashSaveGame(gzFile _gzFile);
+extern void flashReadGame(gzFile _gzFile, int version);
 extern u8 flashRead(u32 address);
 extern void flashWrite(u32 address, u8 byte);
+extern void flashDelayedWrite(u32 address, u8 byte);
 extern u8 flashSaveMemory[0x20000];
 extern void flashSaveDecide(u32 address, u8 byte);
 extern void flashReset();
 extern void flashSetSize(int size);
+extern void flashInit();
 
 extern int flashSize;
 #endif // VBA_FLASH_H
