@@ -1,7 +1,7 @@
 // -*- C++ -*-
 // VisualBoyAdvance - Nintendo Gameboy/GameboyAdvance (TM) emulator.
 // Copyright (C) 1999-2003 Forgotten
-// Copyright (C) 2004 Forgotten and the VBA development team
+// Copyright (C) 2005 Forgotten and the VBA development team
 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -28,43 +28,43 @@
 
 //#define SPRITE_DEBUG
 
-void gfxDrawTextScreen(u16, u16, u16, u32 *);
-void gfxDrawRotScreen(u16,
-                      u16, u16,
-                      u16, u16,
-                      u16, u16,
-                      u16, u16,
-                      int&, int&,
-                      int,
-                      u32*);
-void gfxDrawRotScreen16Bit(u16,
-                           u16, u16,
-                           u16, u16,
-                           u16, u16,
-                           u16, u16,
-                           int&, int&,
-                           int,
-                           u32*);
-void gfxDrawRotScreen256(u16,
-                         u16, u16,
-                         u16, u16,
-                         u16, u16,
-                         u16, u16,
-                         int&, int&,
-                         int,
-                         u32*);
-void gfxDrawRotScreen16Bit160(u16,
-                              u16, u16,
-                              u16, u16,
-                              u16, u16,
-                              u16, u16,
-                              int&, int&,
-                              int,
-                              u32*);
-void gfxDrawSprites(u32 *);
-void gfxIncreaseBrightness(u32 *line, int coeff);
-void gfxDecreaseBrightness(u32 *line, int coeff);
-void gfxAlphaBlend(u32 *ta, u32 *tb, int ca, int cb);
+static void gfxDrawTextScreen(u16, u16, u16, u32 *);
+static void gfxDrawRotScreen(u16,
+			     u16, u16,
+			     u16, u16,
+			     u16, u16,
+			     u16, u16,
+			     int&, int&,
+			     int,
+			     u32*);
+static void gfxDrawRotScreen16Bit(u16,
+				  u16, u16,
+				  u16, u16,
+				  u16, u16,
+				  u16, u16,
+				  int&, int&,
+				  int,
+				  u32*);
+static void gfxDrawRotScreen256(u16,
+				u16, u16,
+				u16, u16,
+				u16, u16,
+				u16, u16,
+				int&, int&,
+				int,
+				u32*);
+static void gfxDrawRotScreen16Bit160(u16,
+				     u16, u16,
+				     u16, u16,
+				     u16, u16,
+				     u16, u16,
+				     int&, int&,
+				     int,
+				     u32*);
+static void gfxDrawSprites(u32 *);
+static void gfxIncreaseBrightness(u32 *line, int coeff);
+static void gfxDecreaseBrightness(u32 *line, int coeff);
+static void gfxAlphaBlend(u32 *ta, u32 *tb, int ca, int cb);
 
 void mode0RenderLine();
 void mode0RenderLineNoWindow();
@@ -100,6 +100,7 @@ extern u32 lineOBJWin[240];
 extern u32 lineMix[240];
 extern bool gfxInWin0[240];
 extern bool gfxInWin1[240];
+extern int lineOBJpixleft[128];
 
 extern int gfxBG2Changed;
 extern int gfxBG3Changed;
@@ -114,15 +115,15 @@ extern int gfxBG3LastX;
 extern int gfxBG3LastY;
 extern int gfxLastVCOUNT;
 
-inline void gfxClearArray(u32 *array)
+static inline void gfxClearArray(u32 *array)
 {
   for(int i = 0; i < 240; i++) {
     *array++ = 0x80000000;
   }
 }
 
-inline void gfxDrawTextScreen(u16 control, u16 hofs, u16 vofs,
-                              u32 *line)
+static inline void gfxDrawTextScreen(u16 control, u16 hofs, u16 vofs,
+				     u32 *line)
 {
   u16 *palette = (u16 *)paletteRAM;
   u8 *charBase = &vram[((control >> 2) & 0x03) * 0x4000];
@@ -266,14 +267,14 @@ inline void gfxDrawTextScreen(u16 control, u16 hofs, u16 vofs,
   }
 }
 
-inline void gfxDrawRotScreen(u16 control, 
-                             u16 x_l, u16 x_h,
-                             u16 y_l, u16 y_h,
-                             u16 pa,  u16 pb,
-                             u16 pc,  u16 pd,
-                             int& currentX, int& currentY,
-                             int changed,
-                             u32 *line)
+static inline void gfxDrawRotScreen(u16 control, 
+				    u16 x_l, u16 x_h,
+				    u16 y_l, u16 y_h,
+				    u16 pa,  u16 pb,
+				    u16 pc,  u16 pd,
+				    int& currentX, int& currentY,
+				    int changed,
+				    u32 *line)
 {
   u16 *palette = (u16 *)paletteRAM;
   u8 *charBase = &vram[((control >> 2) & 0x03) * 0x4000];
@@ -305,7 +306,7 @@ inline void gfxDrawRotScreen(u16 control,
   int dy = pc & 0x7FFF;
   if(pc & 0x8000)
     dy |= 0xFFFF8000;
-  int dmy = pd & 0x7FFFF;
+  int dmy = pd & 0x7FFF;
   if(pd & 0x8000)
     dmy |= 0xFFFF8000;
 
@@ -432,14 +433,14 @@ inline void gfxDrawRotScreen(u16 control,
   }  
 }
 
-inline void gfxDrawRotScreen16Bit(u16 control,
-                                  u16 x_l, u16 x_h,
-                                  u16 y_l, u16 y_h,
-                                  u16 pa,  u16 pb,
-                                  u16 pc,  u16 pd,
-                                  int& currentX, int& currentY,
-                                  int changed,
-                                  u32 *line)
+static inline void gfxDrawRotScreen16Bit(u16 control,
+					 u16 x_l, u16 x_h,
+					 u16 y_l, u16 y_h,
+					 u16 pa,  u16 pb,
+					 u16 pc,  u16 pd,
+					 int& currentX, int& currentY,
+					 int changed,
+					 u32 *line)
 {
   u16 *screenBase = (u16 *)&vram[0];
   int prio = ((control & 3) << 25) + 0x1000000;
@@ -462,7 +463,7 @@ inline void gfxDrawRotScreen16Bit(u16 control,
   int dy = pc & 0x7FFF;
   if(pc & 0x8000)
     dy |= 0xFFFF8000;
-  int dmy = pd & 0x7FFFF;
+  int dmy = pd & 0x7FFF;
   if(pd & 0x8000)
     dmy |= 0xFFFF8000;
 
@@ -529,14 +530,14 @@ inline void gfxDrawRotScreen16Bit(u16 control,
   }  
 }
 
-inline void gfxDrawRotScreen256(u16 control, 
-                                u16 x_l, u16 x_h,
-                                u16 y_l, u16 y_h,
-                                u16 pa,  u16 pb,
-                                u16 pc,  u16 pd,
-                                int &currentX, int& currentY,
-                                int changed,
-                                u32 *line)
+static inline void gfxDrawRotScreen256(u16 control, 
+				       u16 x_l, u16 x_h,
+				       u16 y_l, u16 y_h,
+				       u16 pa,  u16 pb,
+				       u16 pc,  u16 pd,
+				       int &currentX, int& currentY,
+				       int changed,
+				       u32 *line)
 {
   u16 *palette = (u16 *)paletteRAM;
   u8 *screenBase = (DISPCNT & 0x0010) ? &vram[0xA000] : &vram[0x0000];
@@ -560,7 +561,7 @@ inline void gfxDrawRotScreen256(u16 control,
   int dy = pc & 0x7FFF;
   if(pc & 0x8000)
     dy |= 0xFFFF8000;
-  int dmy = pd & 0x7FFFF;
+  int dmy = pd & 0x7FFF;
   if(pd & 0x8000)
     dmy |= 0xFFFF8000;
 
@@ -630,14 +631,14 @@ inline void gfxDrawRotScreen256(u16 control,
   }    
 }
 
-inline void gfxDrawRotScreen16Bit160(u16 control,
-                                     u16 x_l, u16 x_h,
-                                     u16 y_l, u16 y_h,
-                                     u16 pa,  u16 pb,
-                                     u16 pc,  u16 pd,
-                                     int& currentX, int& currentY,
-                                     int changed,
-                                     u32 *line)
+static inline void gfxDrawRotScreen16Bit160(u16 control,
+					    u16 x_l, u16 x_h,
+					    u16 y_l, u16 y_h,
+					    u16 pa,  u16 pb,
+					    u16 pc,  u16 pd,
+					    int& currentX, int& currentY,
+					    int changed,
+					    u32 *line)
 {
   u16 *screenBase = (DISPCNT & 0x0010) ? (u16 *)&vram[0xa000] :
     (u16 *)&vram[0];
@@ -661,7 +662,7 @@ inline void gfxDrawRotScreen16Bit160(u16 control,
   int dy = pc & 0x7FFF;
   if(pc & 0x8000)
     dy |= 0xFFFF8000;
-  int dmy = pd & 0x7FFFF;
+  int dmy = pd & 0x7FFF;
   if(pd & 0x8000)
     dmy |= 0xFFFF8000;
 
@@ -729,8 +730,12 @@ inline void gfxDrawRotScreen16Bit160(u16 control,
   }      
 }
 
-inline void gfxDrawSprites(u32 *lineOBJ)
+static inline void gfxDrawSprites(u32 *lineOBJ)
 {
+  // lineOBJpix is used to keep track of the drawn OBJs
+  // and to stop drawing them if the 'maximum number of OBJ per line'
+  // has been reached.
+  int lineOBJpix = (DISPCNT & 0x20) ? 954 : 1226;
   int m=0;
   gfxClearArray(lineOBJ);
   if(layerEnable & 0x1000) {
@@ -744,55 +749,37 @@ inline void gfxDrawSprites(u32 *lineOBJ)
       u16 a2 = READ16LE(sprites++);
       sprites++;
 
-      // ignore OBJ-WIN
-      if((a0 & 0x0c00) == 0x0800)
-         continue;
-      
-      int sizeY = 8;
-      int sizeX = 8;
-      
-      switch(((a0 >>12) & 0x0c)|(a1>>14)) {
-      case 0:
-        break;
-      case 1:
-        sizeX = sizeY = 16;
-        break;
-      case 2:
-        sizeX = sizeY = 32;
-        break;
-      case 3:
-        sizeX = sizeY = 64;
-        break;
-      case 4:
-        sizeX = 16;
-        break;
-      case 5:
-        sizeX = 32;
-        break;
-      case 6:
-        sizeX = 32;
-        sizeY = 16;
-        break;
-      case 7:
-        sizeX = 64;
-        sizeY = 32;
-        break;
-      case 8:
-        sizeY = 16;
-        break;
-      case 9:
-        sizeY = 32;
-        break;
-      case 10:
-        sizeX = 16;
-        sizeY = 32;
-        break;
-      case 11:
-        sizeX = 32;
-        sizeY = 64;
-        break;
-      default:
+      lineOBJpixleft[x]=lineOBJpix;
+
+      lineOBJpix-=2;
+      if (lineOBJpix<=0)
         continue;
+
+      if ((a0 & 0x0c00) == 0x0c00)
+        a0 &=0xF3FF;
+
+      if ((a0>>14) == 3)
+      {
+        a0 &= 0x3FFF;
+        a1 &= 0x3FFF;
+      }
+      
+      int sizeX = 8<<(a1>>14);
+      int sizeY = sizeX;
+
+      if ((a0>>14) & 1)
+      {
+        if (sizeX<32)
+          sizeX<<=1;
+        if (sizeY>8)
+          sizeY>>=1;
+      }
+      else if ((a0>>14) & 2)
+      {
+        if (sizeX>8)
+          sizeX>>=1;
+        if (sizeY<32)
+          sizeY<<=1;
       }
 
 #ifdef SPRITE_DEBUG
@@ -801,9 +788,45 @@ inline void gfxDrawSprites(u32 *lineOBJ)
 #endif
 
       int sy = (a0 & 255);
+      int sx = (a1 & 0x1FF);
+
+      // computes ticks used by OBJ-WIN if OBJWIN is enabled
+      if (((a0 & 0x0c00) == 0x0800) && (layerEnable & 0x8000))
+      {
+        if ((a0 & 0x0300) == 0x0300)
+        {
+          sizeX<<=1;
+          sizeY<<=1;
+        }
+        if((sy+sizeY) > 256)
+          sy -= 256;
+        if ((sx+sizeX)> 512)
+          sx-=512;
+        if (sx<0)
+        {
+            sizeX+=sx;
+            sx = 0;
+        }
+        else if ((sx+sizeX)>240)
+            sizeX=240-sx;
+        if ((VCOUNT>=sy) && (VCOUNT<sy+sizeY) && (sx<240))
+        {
+          if (a0 & 0x0100)
+            lineOBJpix-=8+2*sizeX;
+          else
+            lineOBJpix-=sizeX-2;
+        }
+        continue;
+      }
+      // else ignores OBJ-WIN if OBJWIN is disabled, and ignored disabled OBJ
+      else
+      if(((a0 & 0x0c00) == 0x0800) || ((a0 & 0x0300) == 0x0200))
+        continue;
+
+      if (lineOBJpix<0)
+        continue;
+
 
-      if(sy > 160)
-        sy -= 256;
       
       if(a0 & 0x0100) {
         int fieldX = sizeX;
@@ -812,11 +835,18 @@ inline void gfxDrawSprites(u32 *lineOBJ)
           fieldX <<= 1;
           fieldY <<= 1;
         }
-        
+        if((sy+fieldY) > 256)
+          sy -= 256;
         int t = VCOUNT - sy;
         if((t >= 0) && (t < fieldY)) {
-          int sx = (a1 & 0x1FF);
-          if((sx < 240) || (((sx + fieldX) & 511) < 240)) {
+          int startpix = 0;
+          if ((sx+fieldX)> 512)
+          {
+            startpix=512-sx;
+          }
+          if (lineOBJpix>0)
+          if((sx < 240) || startpix) {
+            lineOBJpix-=8; 
             // int t2 = t - (fieldY >> 1);
             int rot = (a1 >> 9) & 0x1F;
             u16 *OAM = (u16 *)oam;
@@ -854,6 +884,10 @@ inline void gfxDrawSprites(u32 *lineOBJ)
               else
                 c &= 0x3FE;
               for(int x = 0; x < fieldX; x++) {
+                if (x >= startpix)
+                  lineOBJpix-=2;
+                if (lineOBJpix<0)
+                  continue;
                 int xxx = realX >> 8;
                 int yyy = realY >> 8;
                 
@@ -885,7 +919,7 @@ inline void gfxDrawSprites(u32 *lineOBJ)
                     lineOBJ[sx] = 0x001F;
 #endif
                 }
-                sx = (sx+1)&511;;
+                sx = (sx+1)&511;
                 realX += dx;
                 realY += dy;
               }
@@ -897,8 +931,12 @@ inline void gfxDrawSprites(u32 *lineOBJ)
               int inc = 32;
               if(DISPCNT & 0x40)
                 inc = sizeX >> 3;
-              int palette = (a2 >> 8) & 0xF0;                 
+              int palette = (a2 >> 8) & 0xF0;
               for(int x = 0; x < fieldX; x++) {
+                if (x >= startpix)
+                  lineOBJpix-=2;
+                if (lineOBJpix<0)
+                  continue;
                 int xxx = realX >> 8;
                 int yyy = realY >> 8;
                 if(xxx < 0 || xxx >= sizeX ||
@@ -934,7 +972,7 @@ inline void gfxDrawSprites(u32 *lineOBJ)
                   if(t == 0 || t == maskY || x == 0 || x == maskX)
                     lineOBJ[sx] = 0x001F;
 #endif
-                sx = (sx+1)&511;;
+                sx = (sx+1)&511;
                 realX += dx;
                 realY += dy;
                 
@@ -943,10 +981,17 @@ inline void gfxDrawSprites(u32 *lineOBJ)
           }
         }
       } else {
+        if(sy+sizeY > 256)
+          sy -= 256;
         int t = VCOUNT - sy;
         if((t >= 0) && (t < sizeY)) {
-          int sx = (a1 & 0x1FF);
-          if(((sx < 240)||(((sx+sizeX)&511)<240)) && !(a0 & 0x0200)) {
+          int startpix = 0;
+          if ((sx+sizeX)> 512)
+          {
+            startpix=512-sx;
+          }
+          if((sx < 240) || startpix) {
+            lineOBJpix+=2;
             if(a0 & 0x2000) {
               if(a1 & 0x2000)
                 t = sizeY - t - 1;
@@ -976,6 +1021,10 @@ inline void gfxDrawSprites(u32 *lineOBJ)
               u32 prio = (((a2 >> 10) & 3) << 25) | ((a0 & 0x0c00)<<6);
                 
               for(int xx = 0; xx < sizeX; xx++) {
+                if (xx >= startpix)
+                  lineOBJpix--;
+                if (lineOBJpix<0)
+                  continue;
                 if(sx < 240) {
                   u8 color = vram[address];
                   if ((color==0) && (((prio >> 25)&3) < 
@@ -1048,6 +1097,10 @@ inline void gfxDrawSprites(u32 *lineOBJ)
               if(a1 & 0x1000) {
                 xxx = 7;
                 for(int xx = sizeX - 1; xx >= 0; xx--) {
+                  if (xx >= startpix)
+                    lineOBJpix--;
+                  if (lineOBJpix<0)
+                    continue;
                   if(sx < 240) {
                     u8 color = vram[address];
                     if(xx & 1) {
@@ -1088,6 +1141,10 @@ inline void gfxDrawSprites(u32 *lineOBJ)
                 }           
               } else {        
                 for(int xx = 0; xx < sizeX; xx++) {
+                  if (xx >= startpix)
+                    lineOBJpix--;
+                  if (lineOBJpix<0)
+                    continue;
                   if(sx < 240) {
                     u8 color = vram[address];
                     if(xx & 1) {
@@ -1136,73 +1193,54 @@ inline void gfxDrawSprites(u32 *lineOBJ)
   }
 }
 
-inline void gfxDrawOBJWin(u32 *lineOBJWin)
+static inline void gfxDrawOBJWin(u32 *lineOBJWin)
 {
   gfxClearArray(lineOBJWin);
-  if(layerEnable & 0x8000) {
+  if((layerEnable & 0x9000) == 0x9000) {
     u16 *sprites = (u16 *)oam;
     // u16 *spritePalette = &((u16 *)paletteRAM)[256];
     for(int x = 0; x < 128 ; x++) {
+      int lineOBJpix = lineOBJpixleft[x];
       u16 a0 = READ16LE(sprites++);
       u16 a1 = READ16LE(sprites++);
       u16 a2 = READ16LE(sprites++);
       sprites++;
 
-      // ignore non OBJ-WIN
-      if((a0 & 0x0c00) != 0x0800)
+      if (lineOBJpix<=0)
         continue;
-      
-      int sizeY = 8;
-      int sizeX = 8;
-      
-      switch(((a0 >>12) & 0x0c)|(a1>>14)) {
-      case 0:
-        break;
-      case 1:
-        sizeX = sizeY = 16;
-        break;
-      case 2:
-        sizeX = sizeY = 32;
-        break;
-      case 3:
-        sizeX = sizeY = 64;
-        break;
-      case 4:
-        sizeX = 16;
-        break;
-      case 5:
-        sizeX = 32;
-        break;
-      case 6:
-        sizeX = 32;
-        sizeY = 16;
-        break;
-      case 7:
-        sizeX = 64;
-        sizeY = 32;
-        break;
-      case 8:
-        sizeY = 16;
-        break;
-      case 9:
-        sizeY = 32;
-        break;
-      case 10:
-        sizeX = 16;
-        sizeY = 32;
-        break;
-      case 11:
-        sizeX = 32;
-        sizeY = 64;
-        break;
-      default:
+
+      // ignores non OBJ-WIN and disabled OBJ-WIN
+      if(((a0 & 0x0c00) != 0x0800) || ((a0 & 0x0300) == 0x0200))
         continue;
+
+      if ((a0 & 0x0c00) == 0x0c00)
+        a0 &=0xF3FF;
+      
+      if ((a0>>14) == 3)
+      {
+        a0 &= 0x3FFF;
+        a1 &= 0x3FFF;
+      }
+      
+      int sizeX = 8<<(a1>>14);
+      int sizeY = sizeX;
+
+      if ((a0>>14) & 1)
+      {
+        if (sizeX<32)
+          sizeX<<=1;
+        if (sizeY>8)
+          sizeY>>=1;
+      }
+      else if ((a0>>14) & 2)
+      {
+        if (sizeX>8)
+          sizeX>>=1;
+        if (sizeY<32)
+          sizeY<<=1;
       }
 
       int sy = (a0 & 255);
-
-      if(sy > 160)
-        sy -= 256;
       
       if(a0 & 0x0100) {
         int fieldX = sizeX;
@@ -1211,11 +1249,18 @@ inline void gfxDrawOBJWin(u32 *lineOBJWin)
           fieldX <<= 1;
           fieldY <<= 1;
         }
-        
+        if((sy+fieldY) > 256)
+          sy -= 256;
         int t = VCOUNT - sy;
         if((t >= 0) && (t < fieldY)) {
           int sx = (a1 & 0x1FF);
-          if((sx < 240) || (((sx + fieldX) & 511) < 240)) {
+          int startpix = 0;
+          if ((sx+fieldX)> 512)
+          {
+            startpix=512-sx;
+          }
+          if((sx < 240) || startpix) {
+            lineOBJpix-=8;
             // int t2 = t - (fieldY >> 1);
             int rot = (a1 >> 9) & 0x1F;
             u16 *OAM = (u16 *)oam;
@@ -1249,11 +1294,16 @@ inline void gfxDrawOBJWin(u32 *lineOBJWin)
               else
                 c &= 0x3FE;
               for(int x = 0; x < fieldX; x++) {
+                if (x >= startpix)
+                  lineOBJpix-=2;
+                if (lineOBJpix<0)
+                  continue;
                 int xxx = realX >> 8;
                 int yyy = realY >> 8;
                 
                 if(xxx < 0 || xxx >= sizeX ||
-                   yyy < 0 || yyy >= sizeY) {
+                   yyy < 0 || yyy >= sizeY ||
+                   sx >= 240) {
                 } else {
                   u32 color = vram[0x10000 + ((((c + (yyy>>3) * inc)<<5)
                                     + ((yyy & 7)<<3) + ((xxx >> 3)<<6) +
@@ -1262,7 +1312,7 @@ inline void gfxDrawOBJWin(u32 *lineOBJWin)
                     lineOBJWin[sx] = 1;
                   }
                 }
-                sx = (sx+1)&511;;
+                sx = (sx+1)&511;
                 realX += dx;
                 realY += dy;
               }
@@ -1276,6 +1326,10 @@ inline void gfxDrawOBJWin(u32 *lineOBJWin)
                 inc = sizeX >> 3;
               // int palette = (a2 >> 8) & 0xF0;                      
               for(int x = 0; x < fieldX; x++) {
+                if (x >= startpix)
+                  lineOBJpix-=2;
+                if (lineOBJpix<0)
+                  continue;
                 int xxx = realX >> 8;
                 int yyy = realY >> 8;
 
@@ -1284,7 +1338,8 @@ inline void gfxDrawOBJWin(u32 *lineOBJWin)
                 //                lineOBJ[sx] = 0x001F | prio;
                 //              } else {
                   if(xxx < 0 || xxx >= sizeX ||
-                     yyy < 0 || yyy >= sizeY){
+                     yyy < 0 || yyy >= sizeY ||
+                     sx >= 240) {
                   } else {
                     u32 color = vram[0x10000 + ((((c + (yyy>>3) * inc)<<5)
                                      + ((yyy & 7)<<2) + ((xxx >> 3)<<5) +
@@ -1299,7 +1354,7 @@ inline void gfxDrawOBJWin(u32 *lineOBJWin)
                     }
                   }
                   //            }
-                sx = (sx+1)&511;;
+                sx = (sx+1)&511;
                 realX += dx;
                 realY += dy;
               }       
@@ -1307,10 +1362,18 @@ inline void gfxDrawOBJWin(u32 *lineOBJWin)
           }
         }
       } else {
+        if((sy+sizeY) > 256)
+          sy -= 256;
         int t = VCOUNT - sy;
         if((t >= 0) && (t < sizeY)) {
           int sx = (a1 & 0x1FF);
-          if(((sx < 240)||(((sx+sizeX)&511)<240)) && !(a0 & 0x0200)) {
+          int startpix = 0;
+          if ((sx+sizeX)> 512)
+          {
+            startpix=512-sx;
+          }
+          if((sx < 240) || startpix) {
+            lineOBJpix+=2;
             if(a0 & 0x2000) {
               if(a1 & 0x2000)
                 t = sizeY - t - 1;
@@ -1333,6 +1396,10 @@ inline void gfxDrawOBJWin(u32 *lineOBJWin)
                 xxx = 7;
               // u32 prio = (((a2 >> 10) & 3) << 25) | ((a0 & 0x0c00)<<6);
               for(int xx = 0; xx < sizeX; xx++) {
+                if (xx >= startpix)
+                  lineOBJpix--;
+                if (lineOBJpix<0)
+                  continue;
                 if(sx < 240) {
                   u8 color = vram[address];
                   if(color) {
@@ -1382,6 +1449,10 @@ inline void gfxDrawOBJWin(u32 *lineOBJWin)
               if(a1 & 0x1000) {
                 xxx = 7;
                 for(int xx = sizeX - 1; xx >= 0; xx--) {
+                  if (xx >= startpix)
+                    lineOBJpix--;
+                  if (lineOBJpix<0)
+                    continue;
                   if(sx < 240) {
                     u8 color = vram[address];
                     if(xx & 1) {
@@ -1406,6 +1477,10 @@ inline void gfxDrawOBJWin(u32 *lineOBJWin)
                 }           
               } else {        
                 for(int xx = 0; xx < sizeX; xx++) {
+                  if (xx >= startpix)
+                    lineOBJpix--;
+                  if (lineOBJpix<0)
+                    continue;
                   if(sx < 240) {
                     u8 color = vram[address];
                     if(xx & 1) {
@@ -1437,7 +1512,7 @@ inline void gfxDrawOBJWin(u32 *lineOBJWin)
   }
 }
 
-inline u32 gfxIncreaseBrightness(u32 color, int coeff)
+static inline u32 gfxIncreaseBrightness(u32 color, int coeff)
 {
   int r = (color & 0x1F);
   int g = ((color >> 5) & 0x1F);
@@ -1456,7 +1531,7 @@ inline u32 gfxIncreaseBrightness(u32 color, int coeff)
   return color;
 }
 
-inline void gfxIncreaseBrightness(u32 *line, int coeff)
+static inline void gfxIncreaseBrightness(u32 *line, int coeff)
 {
   for(int x = 0; x < 240; x++) {
     u32 color = *line;
@@ -1477,7 +1552,7 @@ inline void gfxIncreaseBrightness(u32 *line, int coeff)
   }
 }
 
-inline u32 gfxDecreaseBrightness(u32 color, int coeff)
+static inline u32 gfxDecreaseBrightness(u32 color, int coeff)
 {
   int r = (color & 0x1F);
   int g = ((color >> 5) & 0x1F);
@@ -1497,7 +1572,7 @@ inline u32 gfxDecreaseBrightness(u32 color, int coeff)
   return color;
 }
 
-inline void gfxDecreaseBrightness(u32 *line, int coeff)
+static inline void gfxDecreaseBrightness(u32 *line, int coeff)
 {
   for(int x = 0; x < 240; x++) {
     u32 color = *line;
@@ -1518,7 +1593,7 @@ inline void gfxDecreaseBrightness(u32 *line, int coeff)
   }
 }
 
-inline u32 gfxAlphaBlend(u32 color, u32 color2, int ca, int cb)
+static inline u32 gfxAlphaBlend(u32 color, u32 color2, int ca, int cb)
 {
   if(color < 0x80000000) {
     int r = (color & 0x1F);
@@ -1528,10 +1603,10 @@ inline u32 gfxAlphaBlend(u32 color, u32 color2, int ca, int cb)
     int g0 = ((color2 >> 5) & 0x1F);
     int b0 = ((color2 >> 10) & 0x1F);
     
-    r = ((r * ca) >> 4) + ((r0 * cb) >> 4);
-    g = ((g * ca) >> 4) + ((g0 * cb) >> 4);
-    b = ((b * ca) >> 4) + ((b0 * cb) >> 4);
-    
+    r = ((r * ca) + (r0 * cb)) >> 4;
+    g = ((g * ca) + (g0 * cb)) >> 4;
+    b = ((b * ca) + (b0 * cb)) >> 4;
+
     if(r > 31)
       r = 31;
     if(g > 31)
@@ -1544,7 +1619,7 @@ inline u32 gfxAlphaBlend(u32 color, u32 color2, int ca, int cb)
   return color;
 }
 
-inline void gfxAlphaBlend(u32 *ta, u32 *tb, int ca, int cb)
+static inline void gfxAlphaBlend(u32 *ta, u32 *tb, int ca, int cb)
 {
   for(int x = 0; x < 240; x++) {
     u32 color = *ta;
@@ -1557,9 +1632,9 @@ inline void gfxAlphaBlend(u32 *ta, u32 *tb, int ca, int cb)
       int g0 = ((color2 >> 5) & 0x1F);
       int b0 = ((color2 >> 10) & 0x1F);
       
-      r = ((r * ca) >> 4) + ((r0 * cb) >> 4);
-      g = ((g * ca) >> 4) + ((g0 * cb) >> 4);
-      b = ((b * ca) >> 4) + ((b0 * cb) >> 4);
+      r = ((r * ca) + (r0 * cb)) >> 4;
+      g = ((g * ca) + (g0 * cb)) >> 4;
+      b = ((b * ca) + (b0 * cb)) >> 4;
       
       if(r > 31)
         r = 31;
