@@ -26,12 +26,15 @@
 #include "../GBA.h"
 #include "../Globals.h"
 #include "../Text.h"
+#include "../Util.h"
 
 #include "VBA.h"
 #include "MainWnd.h"
 #include "Reg.h"
 #include "resource.h"
 
+#include "Display.h"
+
 #ifdef _DEBUG
 #define new DEBUG_NEW
 #undef THIS_FILE
@@ -39,7 +42,6 @@ static char THIS_FILE[] = __FILE__;
 #endif
 
 extern int Init_2xSaI(u32);
-extern int RGB_LOW_BITS_MASK;
 extern void winlog(const char *,...);
 extern int systemSpeed;
 extern int winVideoModeSelect(CWnd *, GUID **);
@@ -153,7 +155,11 @@ void DirectDrawDisplay::cleanup()
   }
 
   if(ddrawDLL != NULL) {
-    AfxFreeLibrary(ddrawDLL);
+#ifdef _AFXDLL
+    AfxFreeLibrary( ddrawDLL );
+#else
+    FreeLibrary( ddrawDLL );
+#endif
     ddrawDLL = NULL;
   }
   width = 0;
@@ -265,7 +271,12 @@ bool DirectDrawDisplay::initialize()
   if(theApp.pVideoDriverGUID)
     guid = theApp.pVideoDriverGUID;
 
-  ddrawDLL = AfxLoadLibrary("DDRAW.DLL");
+#ifdef _AFXDLL
+  ddrawDLL = AfxLoadLibrary("ddraw.dll");
+#else
+  ddrawDLL = LoadLibrary( _T("ddraw.dll") );
+#endif
+
   HRESULT (WINAPI *DDrawCreateEx)(GUID *,LPVOID *,REFIID,IUnknown *);  
   if(ddrawDLL != NULL) {    
     DDrawCreateEx = (HRESULT (WINAPI *)(GUID *,LPVOID *,REFIID,IUnknown *))
@@ -551,26 +562,22 @@ bool DirectDrawDisplay::initializeOffscreen(int w, int h)
      px.dwBBitMask == 0x001F) {
     systemGreenShift++;
     Init_2xSaI(565);
-    RGB_LOW_BITS_MASK=0x821;
   } else if((px.dwFlags&DDPF_RGB) != 0 &&
             px.dwRBitMask == 0x7C00 &&
             px.dwGBitMask == 0x03E0 &&
             px.dwBBitMask == 0x001F) {
     Init_2xSaI(555);
-    RGB_LOW_BITS_MASK=0x421;
   } else if((px.dwFlags&DDPF_RGB) != 0 &&
             px.dwRBitMask == 0x001F &&
             px.dwGBitMask == 0x07E0 &&
             px.dwBBitMask == 0xF800) {
     systemGreenShift++;
     Init_2xSaI(565);
-    RGB_LOW_BITS_MASK=0x821;
   } else if((px.dwFlags&DDPF_RGB) != 0 &&
             px.dwRBitMask == 0x001F &&
             px.dwGBitMask == 0x03E0 &&
             px.dwBBitMask == 0x7C00) {
     Init_2xSaI(555);
-    RGB_LOW_BITS_MASK=0x421;
   } else {
     // 32-bit or 24-bit
     if(systemColorDepth == 32 || systemColorDepth == 24) {
@@ -588,27 +595,7 @@ bool DirectDrawDisplay::initializeOffscreen(int w, int h)
     winlog("B shift: %d\n", systemBlueShift);
   }
   
-  switch(systemColorDepth) {
-  case 16:
-    {
-      for(int i = 0; i < 0x10000; i++) {
-        systemColorMap16[i] = ((i & 0x1f) << systemRedShift) |
-          (((i & 0x3e0) >> 5) << systemGreenShift) |
-          (((i & 0x7c00) >> 10) << systemBlueShift);
-      }
-    }
-    break;
-  case 24:
-  case 32:
-    {
-      for(int i = 0; i < 0x10000; i++) {
-        systemColorMap32[i] = ((i & 0x1f) << systemRedShift) |
-          (((i & 0x3e0) >> 5) << systemGreenShift) |
-          (((i & 0x7c00) >> 10) << systemBlueShift);
-      }      
-    }
-    break;
-  }
+  utilUpdateSystemColorMaps();
   width = w;
   height = h;
   return true;
@@ -646,16 +633,13 @@ void DirectDrawDisplay::checkFullScreen()
 void DirectDrawDisplay::render()
 {
   HRESULT hret;
+  unsigned int nBytesPerPixel = systemColorDepth>>3;
 
   if(pDirectDraw == NULL ||
      ddsOffscreen == NULL ||
      ddsPrimary == NULL)
     return;
 
-  if(theApp.vsync && !speedup) {
-    hret = pDirectDraw->WaitForVerticalBlank(DDWAITVB_BLOCKBEGIN, 0);
-  }
-  
   DDSURFACEDESC2 ddsDesc;
   
   ZeroMemory(&ddsDesc, sizeof(ddsDesc));
@@ -722,60 +706,7 @@ void DirectDrawDisplay::render()
           copyY = 144;
         }
       }
-      // MMX doesn't seem to be faster to copy the data
-      __asm {
-        mov eax, copyX;
-        mov ebx, copyY;
-        
-        mov esi, pix;
-        mov edi, ddsDesc.lpSurface;
-        mov edx, ddsDesc.lPitch;
-        cmp systemColorDepth, 16;
-        jnz gbaOtherColor;
-        sub edx, eax;
-        sub edx, eax;
-        lea esi,[esi+2*eax+4];
-        shr eax, 1;
-      gbaLoop16bit:
-        mov ecx, eax;
-        repz movsd;
-        inc esi;
-        inc esi;
-        inc esi;
-        inc esi;
-        add edi, edx;
-        dec ebx;
-        jnz gbaLoop16bit;
-        jmp gbaLoopEnd;
-      gbaOtherColor:
-        cmp systemColorDepth, 32;
-        jnz gbaOtherColor2;
-        
-        sub edx, eax;
-        sub edx, eax;
-        sub edx, eax;
-        sub edx, eax;
-        lea esi, [esi+4*eax+4];
-      gbaLoop32bit:
-        mov ecx, eax;
-        repz movsd;
-        add esi, 4;
-        add edi, edx;
-        dec ebx;
-        jnz gbaLoop32bit;
-        jmp gbaLoopEnd;
-      gbaOtherColor2:
-        lea eax, [eax+2*eax];
-        sub edx, eax;
-      gbaLoop24bit:
-        mov ecx, eax;
-        shr ecx, 2;
-        repz movsd;
-        add edi, edx;
-        dec ebx;
-        jnz gbaLoop24bit;
-      gbaLoopEnd:
-      }
+	  copyImage( pix, ddsDesc.lpSurface, copyX, copyY, ddsDesc.lPitch, systemColorDepth );
     }
     if(theApp.showSpeed && (theApp.videoOption > VIDEO_4X || theApp.skin != NULL)) {
       char buffer[30];
@@ -804,6 +735,9 @@ void DirectDrawDisplay::render()
   hret = ddsOffscreen->Unlock(NULL);
 
   if(hret == DD_OK) {
+   if(theApp.vsync && !(theApp.tripleBuffering && theApp.videoOption > VIDEO_4X) && !speedup) { // isn't the Flip() call synced unless a certain flag is passed to it?
+      hret = pDirectDraw->WaitForVerticalBlank(DDWAITVB_BLOCKBEGIN, 0);
+    }
     ddsOffscreen->PageLock(0);
     if(theApp.tripleBuffering && theApp.videoOption > VIDEO_4X) {
       hret = ddsFlip->Blt(&theApp.dest, ddsOffscreen, NULL, DDBLT_WAIT, NULL);
@@ -844,7 +778,7 @@ void DirectDrawDisplay::render()
       SetTextColor(hdc, RGB(255,0,0));
       SetBkMode(hdc,TRANSPARENT);      
       TextOut(hdc, theApp.dest.left+10, theApp.dest.bottom - 20, theApp.screenMessageBuffer,
-              strlen(theApp.screenMessageBuffer));
+              (int)_tcslen(theApp.screenMessageBuffer));
       ddsPrimary->ReleaseDC(hdc);
     } else {
       theApp.screenMessage = false;
