@@ -1,7 +1,7 @@
 // -*- C++ -*-
 // VisualBoyAdvance - Nintendo Gameboy/GameboyAdvance (TM) emulator.
 // Copyright (C) 1999-2003 Forgotten
-// Copyright (C) 2004 Forgotten and the VBA development team
+// Copyright (C) 2005 Forgotten and the VBA development team
 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -55,7 +55,7 @@ class MainWnd : public CWnd
   void winMouseOn();
   void screenCapture(int captureNumber);
   HACCEL m_hAccelTable;
-  bool fileOpenSelect();
+  bool fileOpenSelect( bool gb = false );
   afx_msg LRESULT OnConfirmMode(WPARAM, LPARAM);
   afx_msg LRESULT OnMySysCommand(WPARAM, LPARAM);
   afx_msg void OnUpdateFileLoadGameSlot(CCmdUI *pCmdUI);
@@ -221,14 +221,14 @@ class MainWnd : public CWnd
   afx_msg void OnUpdateOptionsEmulatorPausewheninactive(CCmdUI* pCmdUI);
   afx_msg void OnOptionsEmulatorSpeeduptoggle();
   afx_msg void OnUpdateOptionsEmulatorSpeeduptoggle(CCmdUI* pCmdUI);
-  afx_msg void OnOptionsEmulatorRemoveintrosgba();
-  afx_msg void OnUpdateOptionsEmulatorRemoveintrosgba(CCmdUI* pCmdUI);
   afx_msg void OnOptionsEmulatorAutomaticallyipspatch();
   afx_msg void OnUpdateOptionsEmulatorAutomaticallyipspatch(CCmdUI* pCmdUI);
   afx_msg void OnOptionsEmulatorAgbprint();
   afx_msg void OnUpdateOptionsEmulatorAgbprint(CCmdUI* pCmdUI);
   afx_msg void OnOptionsEmulatorRealtimeclock();
   afx_msg void OnUpdateOptionsEmulatorRealtimeclock(CCmdUI* pCmdUI);
+  afx_msg void OnOptionsEmulatorGenericflashcard();
+  afx_msg void OnUpdateOptionsEmulatorGenericflashcard(CCmdUI* pCmdUI);
   afx_msg void OnOptionsEmulatorAutohidemenu();
   afx_msg void OnUpdateOptionsEmulatorAutohidemenu(CCmdUI* pCmdUI);
   afx_msg void OnOptionsEmulatorRewindinterval();
@@ -408,13 +408,20 @@ class MainWnd : public CWnd
   afx_msg void OnUpdateOptionsSoundVolume5x(CCmdUI* pCmdUI);
   afx_msg void OnCheatsDisablecheats();
   afx_msg void OnUpdateCheatsDisablecheats(CCmdUI* pCmdUI);
-	afx_msg void OnOptionsVideoFullscreenmaxscale();
+  afx_msg void OnOptionsVideoFullscreenmaxscale();
+  afx_msg void OnOptionsEmulatorGameoverrides();
+  afx_msg void OnUpdateOptionsEmulatorGameoverrides(CCmdUI* pCmdUI);
+	afx_msg void OnHelpGnupubliclicense();
 	//}}AFX_MSG
   DECLARE_MESSAGE_MAP()
-
+    
     afx_msg BOOL OnFileRecentFile(UINT nID);
   afx_msg BOOL OnFileLoadSlot(UINT nID);
   afx_msg BOOL OnFileSaveSlot(UINT nID);
+public:
+	afx_msg void OnOptionsSoundHardwareacceleration();
+public:
+	afx_msg void OnUpdateOptionsSoundHardwareacceleration(CCmdUI *pCmdUI);
 };
 
 /////////////////////////////////////////////////////////////////////////////
